from fastapi.exceptions import HTTPException

class Permission:

    """
    r -> read -> 4 = GET
    w -> write -> 2 = PUT
    d -> delete -> 1 = DELETE

    0 = ---
    1 = --d
    2 = -w-
    3 = -wd
    4 = r--
    5 = r-d
    6 = rw-
    7 = rwd
    """

    def __init__(self, operation, user_uuid, resource):
        self.operation = operation
        self.user_uuid = user_uuid
        self.resource = resource

    async def _get_perms(operation, perms, index):

        operation = int(operation)
        perms_for_user = int(str(perms)[index])

        if operation == 4 and perms_for_user in [4,5,6,7]:
            return True

        elif operation == 2 and perms_for_user in [2,3,6,7]:
            return True

        elif operation == 1 and perms_for_user in [1,3,5,7]:
            return True

        else:
            raise HTTPException(403)


    async def _is_allowed(self, 
        operation = 0, 
        perms = 000, 
        user_uuid = None, 
        owner = None, 
        group = []):

        if  user_uuid == owner:
            return await self._get_perms(operation , perms, 0)

        elif user_uuid in group:
            return await self._get_perms(operation , perms, 1)

        elif user_uuid not in group:
            return await self._get_perms(operation , perms, 2)


    async def get_verdict(self):

        if isinstance(self.operation, str):
            http_perms_key = {
                "GET": 4,
                "PUT": 2,
                "DELETE": 1
            }
            self.operation = http_perms_key[self.operation]
            
        return await self._is_allowed(self.operation, self.resource.permissions, self.user_uuid, self.resource.owner, self.resource.group)



from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = '0.0.4'
DESCRIPTION = 'Test_pkg'
LONG_DESCRIPTION = 'A package that allows to build simple microservices with fastapi and mongoDB'

# Setting up
setup(
    name="fmfancy",
    version=VERSION,
    author="Tobiasz Gleba",
    author_email="tobiasz.gleba24@gmail.com",
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    install_requires=['fastapi', 'motor'],
    keywords=['python', 'fastapi', 'workflow'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6"
)


from fastapi.exceptions import HTTPException
import motor.motor_asyncio

class Database():

    def __init__(self, db_url, db_name):
        self.client = motor.motor_asyncio.AsyncIOMotorClient(db_url)
        self.db = self.client[db_name]


    async def db_check_connection(self) -> int:

        try:
            await self.client.admin.command("serverStatus")
            await self.db.command('ping')
            return 200
            
        except Exception:
            return 503

    async def insert_one(self, pydantic_object):

        content_dict = dict(pydantic_object)
        collection = pydantic_object.__class__.__name__

        await self.db[collection].insert_one(content_dict)

    async def find_one(self, pydantic_object):

        content_dict = dict(pydantic_object)
        collection = pydantic_object.__class__.__name__
        # find_one({"_id": new_student.inserted_id})
        
        if (query_result := await self.db[collection].find_one(content_dict, )) is not None:
            return query_result
        else:
            raise HTTPException(404)
    

    def shutdown_database(self):
        self.client.close()


    def __del__(self):
        self.shutdown_database()